from .pornemby_double import PornembyDoubleMonitor

__ignore__ = True


class TestPornembyDoubleMonitor(PornembyDoubleMonitor):
    name = "Pornemby 怪兽自动翻倍 测试"
    chat_name = "api_group"
    chat_user = "embykeeper_test_bot"
    chat_allow_outgoing = True
